import 'package:flutter/material.dart';

///Animation Types
enum AnimationType {
  normal,
  fadeIn,
}

/// Main Class, [context] is the BuildContext in the app
/// [child] is the widget to transition
/// [animation] is the type of animation
class RouteTransitions {
  final BuildContext context;
  final Widget child;
  final AnimationType animation;
  final Duration duration;
  final bool replacement;

  RouteTransitions({
    @required this.context,
    @required this.child,
    this.replacement = false,
    this.animation = AnimationType.normal,
    this.duration = const Duration(milliseconds: 300),
  }) {
    switch (animation) {
      case AnimationType.normal:
        _normalTransition();
        break;
      case AnimationType.fadeIn:
        _fadeInTransition();
        break;
      default:
    }
  }

  /// Normal push of the page
  void _pushPage(Route route) => Navigator.push(context, route);

  /// Replacement push of the page
  void _pushReplacementPage(Route route) =>
      Navigator.pushReplacement(context, route);

  /// Normal transition method
  void _normalTransition() {
    final route = MaterialPageRoute(builder: (_) => this.child);
    replacement ? _pushReplacementPage(route) : _pushPage(route);
  }

  /// Controller of the FadeIn transition
  void _fadeInTransition() {
    final route = PageRouteBuilder(
      pageBuilder: (_, __, ___) => child,
      transitionDuration: this.duration,
      transitionsBuilder: (_, animation, __, child) => FadeTransition(
        opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
          CurvedAnimation(
            parent: animation,
            curve: Curves.easeOut,
          ),
        ),
        child: child,
      ),
    );
    replacement ? _pushReplacementPage(route) : _pushPage(route);
  }
}
